package App::ABAN::BeepChild;
#ABSTRACT: ABAN internal POE session to beep a disk count
use 5.028;
use warnings qw(all);
use strict;

use POE qw(Wheel::Run);

sub create {
	my (undef, $ident, $disk_count) = @_;

	my @cmd;
	my %handlers = (
		_start       => 'start_session',
		child_signal => 'child_signal',
	);

	return POE::Session->create(
		args => [ $ident, $disk_count ],
		package_states => [ __PACKAGE__, \%handlers ]
	);
}

sub start_session {
	$_[HEAP]{ident} = $_[ARG0];
	$_[KERNEL]->alias_set("BeepChild-$_[HEAP]{ident}");

	# beep refuses to run since ABAN obtained root with sudo. So hide
	# that fact from it.
	my @cmd = (
		qw(env -u SUDO_COMMAND -u SUDO_USER -u SUDO_UID -u SUDO_GID
			beep -f 1568 -l 250 -D 400 -r), $_[ARG1]);
	my $child = POE::Wheel::Run->new(
		Program => \@cmd,
		NoStdin => 1,
	);
	$_[KERNEL]->sig_child($child->PID, 'child_signal');
	$_[HEAP]{child} = $child;

	return;
}

sub child_signal { 
	# remove both refs to allow GC
	$_[HEAP]{child} = undef; 
	$_[KERNEL]->alias_remove("WipeChild-$_[HEAP]{ident}");
}

1;

__END__

=pod

=encoding UTF-8

=head1 NAME

App::ABAN::BeepChild - ABAN internal POE session to beep a disk count

=head1 VERSION

version 0.020

=head1 DESCRIPTION

This module is a L<POE::Session> to run the "beep" command to indicate
how many disks we're wiping. You determine the number of disks by
counting the beeps. This is to allow headless operation.

=head1 SEE ALSO

=over 4

=item L<ABAN>

For detail about what ABAN is, see the main L<ABAN> manual page/perldoc.

=back

=head1 AUTHOR

Anthony DeRobertis <anthony@derobert.net>

=head1 COPYRIGHT AND LICENSE

Copyright (C)2019-2020 Customer Relationship Metrics.
Copyright (C)2019-2022 Anthony DeRobertis.
Web page modern style (C)2020 Ruben Albornoz.

This program is free software; you can redistribute it and/or modify it
under the terms of the Perl Artistic License or the GNU General Public
License as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

In addition, if you are distributing under the GPL, as a special
exception, the above copyright holders give permission to link the code
of this program with the OpenSSL Library (or with modified versions of
OpenSSL that use the same license as OpenSSL), and distribute linked
combinations including the two. You must obey the GNU General Public
License in all respects for all of the code used other than OpenSSL.

This exception applies to versions of OpenSSL under the traditional
(pre-3.0) OpenSSL licensing, versions under Apache License v2, and any
future license OpenSSL may be released under.

If you modify this program, you may extend this exception to your
version, but you are not obligated to do so. If you do not wish to do
so, delete this exception statement from your version.

=cut
